@@ -55,6 +55,10 @@ static unsigned int debug;
 module_param(debug, uint, 0644);
 MODULE_PARM_DESC(debug, "1=err, 2=isr, 3=xfer");
 
+static unsigned int clk_tout_ms = 35; /* SMBUs-recommended 35ms */
+module_param(clk_tout_ms, uint, 0644);
+MODULE_PARM_DESC(clk_tout_ms, "clock-stretch timeout (mS)");
+
 #define BCM2835_DEBUG_MAX	512
 struct bcm2835_debug {
 	struct i2c_msg *msg;
@@ -214,12 +218,12 @@ static int clk_bcm2835_i2c_set_rate(struct clk_hw *hw, unsigned long rate,
 			   (redl << BCM2835_I2C_REDL_SHIFT));
 
 	/*
-	 * Set the clock stretch timeout to the SMBUs-recommended 35ms.
+	 * Set the clock stretch timeout.
 	 */
-	if (rate > 0xffff*1000/35)
+	if (rate > 0xffff*1000/clk_tout_ms)
 	    clk_tout = 0xffff;
 	else
-	    clk_tout = 35*rate/1000;
+	    clk_tout = clk_tout_ms*rate/1000;
 
 	bcm2835_i2c_writel(div->i2c_dev, BCM2835_I2C_CLKT, clk_tout);
 
@@ -373,10 +377,6 @@ static irqreturn_t bcm2835_i2c_isr(int this_irq, void *data)
 	u32 val, err;
 
 	val = bcm2835_i2c_readl(i2c_dev, BCM2835_I2C_S);
-	bcm2835_i2c_writel(i2c_dev, BCM2835_I2C_S,
-			   BCM2835_I2C_S_CLKT |
-			   BCM2835_I2C_S_ERR |
-			   BCM2835_I2C_S_DONE);
 	bcm2835_debug_add(i2c_dev, val);
 
 	err = val & (BCM2835_I2C_S_CLKT | BCM2835_I2C_S_ERR);
@@ -386,9 +386,6 @@ static irqreturn_t bcm2835_i2c_isr(int this_irq, void *data)
 	}
 
 	if (val & BCM2835_I2C_S_DONE) {
-		if (val & BCM2835_I2C_S_TA)
-			return IRQ_HANDLED;
-
 		if (!i2c_dev->curr_msg) {
 			dev_err(i2c_dev->dev, "Got unexpected interrupt (from firmware?)\n");
 		} else if (i2c_dev->curr_msg->flags & I2C_M_RD) {
@@ -396,10 +393,7 @@ static irqreturn_t bcm2835_i2c_isr(int this_irq, void *data)
 			val = bcm2835_i2c_readl(i2c_dev, BCM2835_I2C_S);
 		}
 
-		if (i2c_dev->msg_buf_remaining)
-			return IRQ_HANDLED;
-
-		if (val & BCM2835_I2C_S_RXD)
+		if ((val & BCM2835_I2C_S_RXD) || i2c_dev->msg_buf_remaining)
 			i2c_dev->msg_err = BCM2835_I2C_S_LEN;
 		else
 			i2c_dev->msg_err = 0;
@@ -436,6 +430,8 @@ static irqreturn_t bcm2835_i2c_isr(int this_irq, void *data)
 
 complete:
 	bcm2835_i2c_writel(i2c_dev, BCM2835_I2C_C, BCM2835_I2C_C_CLEAR);
+	bcm2835_i2c_writel(i2c_dev, BCM2835_I2C_S, BCM2835_I2C_S_CLKT |
+			   BCM2835_I2C_S_ERR | BCM2835_I2C_S_DONE);
 	complete(&i2c_dev->completion);
 
 	return IRQ_HANDLED;
